// Written by Jason Heddings: http://mapit.mozdev.org/
// License: http://creativecommons.org/licenses/by-nc-sa/2.0/

var gMapItAddress = null;
var gMapItSettings = null;
var gMapItDirections = 0;

////////////////////////////////////////////////////////////////////////////////
// Attempt to read address from selected text.

// var directions: indicates if the map should be driving directions
// 0 - do not show directions
// 1 - show directions to location from saved address
// 2 - show directions from location to saved address
function mapitSelectedText(directions) {
  if (gMapItAddress == null) {
    alert("how'd this happen?");
  }

  gMapItDirections = directions;

  if ((gMapItSettings.verify == "always") || (! gMapItAddress.valid)) {
    // display the dialog to correct the address...
    window.open("chrome://mapit/content/mapitDialog.xul", "_blank", "chrome,resizable");
  } else {
    // submit the address to the selected map engine
    mapitAddress(gMapItAddress);
  }
}

////////////////////////////////////////////////////////////////////////////////
// Map a specific address.

function mapitAddress(address) {
  if (! address.valid) {
    return;
  }

  // set up the default second address (if we should)
  var dfault = null;
  if (gMapItSettings.directions == "home") {
    dfault = new MapItAddress();
    dfault.street = gMapItSettings.homeStreet;
    dfault.city = gMapItSettings.homeCity;
    dfault.zip = gMapItSettings.homeZip;
    dfault.state = gMapItSettings.homeState;
  }

  // figure out which is the to & from addresses
  var to = (gMapItDirections == 1) ? address : dfault;
  var from = (gMapItDirections == 2) ? address : dfault;

  var form = (gMapItDirections == 0) 
           ? mapitGetShowMapForm(address)
           : mapitGetDirectionsForm(to, from);

  if (form != null) {
    mapitSubmit(form);
  }
}

////////////////////////////////////////////////////////////////////////////////
// Creates a new MapItAddress from the currently selected text.

function mapitReadAddress() {
  var addr = window._content.__proto__.getSelection.call(window._content).toString();
  return new MapItAddress(addr);
}

////////////////////////////////////////////////////////////////////////////////
// Called when dialog popup is displayed.

function showMapItPopup(isFromMapItMenu) {
  if (gMapItSettings == null) {
    gMapItSettings = new MapItSettings();
  }

  if (isFromMapItMenu) {
    // parse the highlighted text for an address
    gMapItAddress = mapitReadAddress();
  } else if (gMapItSettings.verify == "never") {
    // remove the mapit options from the menu if they shouldn't be there
    gMapItAddress = mapitReadAddress();
    document.getElementById("mapit-context-menu").hidden = (! gMapItAddress.valid);
  }
}

////////////////////////////////////////////////////////////////////////////////
// Called when dialog popup is hidden.

function hideMapItPopup(isFromMapItMenu) {
}

////////////////////////////////////////////////////////////////////////////////
// Change full state name to abbreviated one.
// TODO :jah: variants of names should be considered

function mapitAbbreviateState(state) {
  if (! state) {
    return null;
  }

  switch (state.toLowerCase()) {
    case "alabama":        return "AL";
    case "alaska":         return "AK";
    case "arizona":        return "AZ";
    case "arkansas":       return "AR";
    case "california":     return "CA";
    case "colorado":       return "CO";
    case "connecticut":    return "CT";
    case "d.c.":           return "DC";
    case "delaware":       return "DE";
    case "florida":        return "FL";
    case "georgia":        return "GA";
    case "hawaii":         return "HI";
    case "idaho":          return "ID";
    case "illinois":       return "IL";
    case "indiana":        return "IN";
    case "iowa":           return "IA";
    case "kansas":         return "KS";
    case "kentucky":       return "KY";
    case "louisiana":      return "LA";
    case "maine":          return "ME";
    case "maryland":       return "MD";
    case "massachusetts":  return "MA";
    case "michigan":       return "MI";
    case "minnesota":      return "MN";
    case "mississippi":    return "MS";
    case "missouri":       return "MO";
    case "montana":        return "MT";
    case "nebraska":       return "NE";
    case "nevada":         return "NV";
    case "new hampshire":  return "NH";
    case "new jersey":     return "NJ";
    case "new mexico":     return "NM";
    case "new york":       return "NY";
    case "north carolina": return "NC";
    case "north dakota":   return "ND";
    case "ohio":           return "OH";
    case "oklahoma":       return "OK";
    case "oregon":         return "OR";
    case "pennsylvania":   return "PA";
    case "rhode island":   return "RI";
    case "south carolina": return "SC";
    case "south dakota":   return "SD";
    case "tennessee":      return "TN";
    case "texas":          return "TX";
    case "utah":           return "UT";
    case "vermont":        return "VT";
    case "virginia":       return "VA";
    case "washington":     return "WA";
    case "west virginia":  return "WV";
    case "wisconsin":      return "WI";
    case "wyoming":        return "WY";
  }

  return state;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

function MapItAddress(str) { this._init(str); }

////////////////////////////////////////////////////////////////////////////////

MapItAddress.prototype = {
  _raw:null,

  valid:null,

  street:null,
  city:null,
  state:null,
  zip:null,
  country:null,

_console:null,

////////////////////////////////////////////////////////////////////////////////

  validate:function() {
    this.valid = true;
  },

////////////////////////////////////////////////////////////////////////////////

  _init:function(str) {
    this._console = Components.classes["@mozilla.org/consoleservice;1"]
                    .getService(Components.interfaces.nsIConsoleService);

    this._parse(str);

    // TODO :jah: make this work for other countries
    this.country = "US";
  },

////////////////////////////////////////////////////////////////////////////////

  _parse:function(str) {
    this._raw = str;
    this.valid = false;

    if (str) {
      // try to match the address to an existing regular expression
      var regexps = gMapItSettings.regexps;
      for (var idx = 0; idx < regexps.length; idx++) {
        var match = (new RegExp(regexps[idx])).exec(this._raw, "m");
        if (match != null) {
          this.street = this._clean(match[1]);
          this.city = this._clean(match[2]);
          this.state = this._clean(match[3]);
          this.zip = this._clean(match[4]);
          this.valid = true;
          break;
        }
      }
    }
  },

////////////////////////////////////////////////////////////////////////////////

  _clean:function(str) {
    return str.replace(/[^A-Za-z0-9 .,-]/g, "");
  },

////////////////////////////////////////////////////////////////////////////////

  toString:function() {
    return this.street + "\n"
           + this.city + ", " + this.state + " " + this.zip + "\n"
           + this.country;
  },

////////////////////////////////////////////////////////////////////////////////
};
