// Written by Jason Heddings: http://mapit.mozdev.org/
// License: http://creativecommons.org/licenses/by-nc-sa/2.0/

// Here are some links that I'd like to incorporate into the extension...
// http://www.mappoint.com
// http://www.randmcnally.com/rmc/directions/dirGetMapInput.jsp

var gMapItDocument;

////////////////////////////////////////////////////////////////////////////////
// Generates a page in a new tab or window

function mapitGenerateDocument(url) {
  var settings = new MapItSettings();
  var genPage = window;

  // check where to send the map
  if (settings.target == "tab") {
    getBrowser().selectedTab = getBrowser().addTab(url);
  } else if (settings.target == "blank") {
    genPage = window.open(url);
  } else {
    // the new page is set to the current window
  }

  return genPage.content.document;
}

////////////////////////////////////////////////////////////////////////////////
// Creates a blank form.

function mapitGenerateForm(method, action) {
  gMapItDocument = mapitGenerateDocument("about:blank");
  var form = gMapItDocument.createElement("form")
  form.setAttribute("method", method);
  form.setAttribute("action", action);
  gMapItDocument.body.appendChild(form);
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Creates a form input element.

function mapitGenerateInput(name, value) {
  var input = gMapItDocument.createElement("input");
  input.setAttribute("type", "hidden");
  input.setAttribute("name", name);
  input.setAttribute("value", value);
  return input;
}

////////////////////////////////////////////////////////////////////////////////
// Get the form to show directions from ons address to another.

function mapitGetDirectionsForm(to, from) {
  switch (gMapItSettings.mapper) {
    case "mapsdotcom": return mapitMapsDotComDirections(to, from); break;
    //case "maporama":   return mapitMaporamaDirections(to, from); break;
    case "mapsonus":   return mapitMapsOnUsDirections(to, from); break;
    case "yahoo":      return mapitYahooDirections(to, from); break;

    default:           return mapitMapQuestDirections(to, from); break;
  }
}

////////////////////////////////////////////////////////////////////////////////
// Get the form to show a specific map.

function mapitGetShowMapForm(address) {
  switch (gMapItSettings.mapper) {
    case "geocode":      return mapitGeocode(address); break;
    case "globexplorer": return mapitGlobeXplorer(address); break;
    case "mapsdotcom":   return mapitMapsDotCom(address); break;
    case "maporama":     return mapitMaporama(address); break;
    case "mapsonus":     return mapitMapsOnUs(address); break;
    case "terraserver":  return mapitTerraServer(address); break;
    case "yahoo":        return mapitYahoo(address); break;

    default:             return mapitMapQuest(address); break;
  }
}

////////////////////////////////////////////////////////////////////////////////
// Submits the form.

function mapitSubmit(form) {
  // This must be done to make generated content render
  const request = new XMLHttpRequest();
  request.open("GET", window.content.document.URL, true);
  request.send("");

  form.submit();
}

////////////////////////////////////////////////////////////////////////////////
// Create the MapQuest form.
// http://www.mapquest.com/

function mapitMapQuest(address) {
  var form = mapitGenerateForm("get", "http://www.mapquest.com/maps/map.adp");
  form.appendChild(mapitGenerateInput("CID", "lfmapwid"));
  form.appendChild(mapitGenerateInput("country", address.country));
  form.appendChild(mapitGenerateInput("address", address.street));
  form.appendChild(mapitGenerateInput("city", address.city));
  //form.appendChild(mapitGenerateInput("state", mapitAbbreviateState(address.state)));
  form.appendChild(mapitGenerateInput("zipcode", address.zip));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the MapQuest form for finding directions.
// http://www.mapquest.com/

function mapitMapQuestDirections(to, from) {
  var form = mapitGenerateForm("get", "http://www.mapquest.com/directions/main.adp");

  if ((from != null) && (to != null)) {
    form.appendChild(mapitGenerateInput("go", "1"));
  }

  if (from != null) {
    form.appendChild(mapitGenerateInput("1y", from.country));
    form.appendChild(mapitGenerateInput("1a", from.street));
    form.appendChild(mapitGenerateInput("1c", from.city));
    //form.appendChild(mapitGenerateInput("1s", mapitAbbreviateState(from.state)));
    form.appendChild(mapitGenerateInput("1z", from.zip));
  }

  if (to != null) {
    form.appendChild(mapitGenerateInput("2y", to.country));
    form.appendChild(mapitGenerateInput("2a", to.street));
    form.appendChild(mapitGenerateInput("2c", to.city));
    //form.appendChild(mapitGenerateInput("2s", mapitAbbreviateState(to.state)));
    form.appendChild(mapitGenerateInput("2z", to.zip));
  }

  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Maps.com form.
// http://www.maps.com/directions/address/simple_address.html

function mapitMapsDotCom(address) {
  var form = mapitGenerateForm("post", "http://host.maptuit.com/maps.com/index.cgi");
  form.appendChild(mapitGenerateInput("AddrPickBest", "1"));
  form.appendChild(mapitGenerateInput("Address", address.street));
  form.appendChild(mapitGenerateInput("City", address.city));
  form.appendChild(mapitGenerateInput("Region", mapitAbbreviateState(address.state)));
  form.appendChild(mapitGenerateInput("Postal", address.zip));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Maps.com form for finding directions.
// http://www.maps.com/

function mapitMapsDotComDirections(to, from) {
  var form = mapitGenerateForm("post", "http://host.maptuit.com/maps.com/index.cgi");

  if (from != null) {
    form.appendChild(mapitGenerateInput("Address_0", from.street));
    form.appendChild(mapitGenerateInput("City_0", from.city));
    form.appendChild(mapitGenerateInput("Region_0", mapitAbbreviateState(from.state)));
    form.appendChild(mapitGenerateInput("Postal_0", from.zip));
  }

  if (to != null) {
    form.appendChild(mapitGenerateInput("Address_1", to.street));
    form.appendChild(mapitGenerateInput("City_1", to.city));
    form.appendChild(mapitGenerateInput("Region_1", mapitAbbreviateState(to.state)));
    form.appendChild(mapitGenerateInput("Postal_1", to.zip));
  }

  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the GlobeXplorer form.
// http://www.globexplorer.com/

function mapitGlobeXplorer(address) {
  var form = mapitGenerateForm("post", "http://imageatlas.globexplorer.com/ImageAtlas/view.do?group=ImageAtlas");
  form.appendChild(mapitGenerateInput("address", address.street));
  form.appendChild(mapitGenerateInput("city", address.city));
  form.appendChild(mapitGenerateInput("state", mapitAbbreviateState(address.state)));
  form.appendChild(mapitGenerateInput("zip", address.zip));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the TerraServer form.
// http://terraserver.microsoft.com/address.aspx

function mapitTerraServer(address) {
  var form = mapitGenerateForm("post", "http://terraserver.microsoft.com/address.aspx");
  form.appendChild(mapitGenerateInput("PgSrh:NavScalePS", ""));
  form.appendChild(mapitGenerateInput("PgSrh:NavWPS", ""));
  form.appendChild(mapitGenerateInput("PgSrh:NavStreet", address.street));
  form.appendChild(mapitGenerateInput("PgSrh:NavCity", address.city));
  form.appendChild(mapitGenerateInput("PgSrh:NavState", mapitAbbreviateState(address.state)));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Yahoo! form.
// http://maps.yahoo.com

function mapitYahoo(address) {
  var form = mapitGenerateForm("get", "http://us.rd.yahoo.com/maps/home/submit_a/*-http://maps.yahoo.com/maps");
  form.appendChild(mapitGenerateInput("newFL", "Use Address Below"));
  form.appendChild(mapitGenerateInput("addr", address.street));
  form.appendChild(mapitGenerateInput("csz", address.city + ", " + address.state + ", " + address.zip));
  form.appendChild(mapitGenerateInput("country", address.country.toLowerCase()));
  form.appendChild(mapitGenerateInput("name", ""));
  form.appendChild(mapitGenerateInput("lat", ""));
  form.appendChild(mapitGenerateInput("lon", ""));
  form.appendChild(mapitGenerateInput("qty", ""));
  form.appendChild(mapitGenerateInput("srchtype", "a"));
  form.appendChild(mapitGenerateInput("getmap", "Get Map"));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Yahoo! form for finding directions.
// http://maps.yahoo.com/

function mapitYahooDirections(to, from) {
  var form = mapitGenerateForm("get", "http://us.rd.yahoo.com/maps/mapsdd/submit/*-http://maps.yahoo.com/dd_result");

  if (from != null) {
    form.appendChild(mapitGenerateInput("addr", from.street));
    form.appendChild(mapitGenerateInput("csz", from.city + ", " + from.state + ", " + from.zip));
    form.appendChild(mapitGenerateInput("country", from.country.toLowerCase()));
  }

  if (to != null) {
    form.appendChild(mapitGenerateInput("taddr", to.street));
    form.appendChild(mapitGenerateInput("tcsz", to.city + ", " + to.state + ", " + to.zip));
    form.appendChild(mapitGenerateInput("tcountry", to.country.toLowerCase()));
  }

  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Geocode (Eagle) form.
// http://www.geocode.com

function mapitGeocode(address) {
  var form = mapitGenerateForm("get", "http://www.geocode.com/modules.php");
  form.appendChild(mapitGenerateInput("street", address.street));
  form.appendChild(mapitGenerateInput("city", address.city));
  form.appendChild(mapitGenerateInput("state1", address.state));
  form.appendChild(mapitGenerateInput("state", mapitAbbreviateState(address.state)));
  form.appendChild(mapitGenerateInput("zip", address.zip));
  form.appendChild(mapitGenerateInput("name", "TestDrive_Eagle"));
  form.appendChild(mapitGenerateInput("dummy", "dummy"));
  form.appendChild(mapitGenerateInput("dum2", "dummy"));
  form.appendChild(mapitGenerateInput("request", "Address & Map"));
  form.appendChild(mapitGenerateInput("geocode", "request"));
  form.appendChild(mapitGenerateInput("cmd", "td_g"));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the MapsOnUs Form
// http://www.mapsonus.com

function mapitMapsOnUs(address) {
  var form = mapitGenerateForm("post", "http://www.mapsonus.com/bin/maps-mapspec/usr=~new/formName=mapsetaddr/p=m65");
  form.appendChild(mapitGenerateInput("AddrStreetLine", address.street));
  form.appendChild(mapitGenerateInput("AddrCityLine", address.city + ", " + address.state + ", " + address.zip));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the MapsOnUs directions Form
// http://www.mapsonus.com

function mapitMapsOnUsDirections(to, from) {
  var form = mapitGenerateForm("post", "http://www.mapsonus.com/bin/maps-planroute/usr=~new/formName=mapsetaddr/p=m65");
  form.appendChild(mapitGenerateInput("AddMarker", "Marker=S"));
  form.appendChild(mapitGenerateInput("AddMarker", "Marker=E"));
  form.appendChild(mapitGenerateInput("SelectItems", "S E"));
  form.appendChild(mapitGenerateInput("AddrPickBest", "0"));
  form.appendChild(mapitGenerateInput("RouteType", "f"));

  if (from != null) {
    form.appendChild(mapitGenerateInput("AddrStreetLine_S", from.street));
    form.appendChild(mapitGenerateInput("AddrCityLine_S", from.city + ", " + from.state + ", " + from.zip));
  }

  if (to != null) {
    form.appendChild(mapitGenerateInput("AddrStreetLine_E", to.street));
    form.appendChild(mapitGenerateInput("AddrCityLine_E", to.city + ", " + to.state + ", " + to.zip));
  }

  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Maporama Form
// http://www.maporama.com

function mapitMaporama(address) {
  var form = mapitGenerateForm("get", "http://maporama.com/share/map.asp");
  form.appendChild(mapitGenerateInput("_XgoGCAddress", address.street));
  form.appendChild(mapitGenerateInput("Zip", address.zip));
  form.appendChild(mapitGenerateInput("State", address.state));
  form.appendChild(mapitGenerateInput("_XgoGCTownName", address.city));
  form.appendChild(mapitGenerateInput("COUNTRYCODE", address.country));
  return form;
}

////////////////////////////////////////////////////////////////////////////////
// Create the Maporama directions Form
// http://www.maporama.com

function mapitMaporamaDirections(to, from) {
  var form = mapitGenerateForm("get", "http://maporama.com/share/iti.asp");
  form.appendChild(mapitGenerateInput("ITI_MODE", "vehicule"));
  form.appendChild(mapitGenerateInput("ITI_MODE_OPTION_vehicule", "Time"));

  if (from != null) {
    form.appendChild(mapitGenerateInput("ITI_START_COUNTRYCODE", from.country));
    form.appendChild(mapitGenerateInput("ITI_START_ADDRESS", from.street));
    form.appendChild(mapitGenerateInput("ITI_START_ZIPCODE", from.zip));
    form.appendChild(mapitGenerateInput("ITI_START_STATE", from.state));
    form.appendChild(mapitGenerateInput("ITI_START_CITYNAME", from.city));
  }

  if (to != null) {
    form.appendChild(mapitGenerateInput("ITI_END_COUNTRYCODE", to.country));
    form.appendChild(mapitGenerateInput("ITI_END_ADDRESS", to.street));
    form.appendChild(mapitGenerateInput("ITI_END_ZIPCODE", to.zip));
    form.appendChild(mapitGenerateInput("ITI_END_STATE", to.state));
    form.appendChild(mapitGenerateInput("ITI_END_CITYNAME", to.city));
  }

  return form;
}
